
package com.jmrosengard.xmlbroker.broker;

import java.net.*;
import java.io.*;

/**
* A trivial HTTP client. Only (some of) the POST method is
* implemented, using java.net.Socket. Note that the higher-level java.net.URL
* object do <i>not</i> fit into the SOAP model, and is therefore not used.
*/

public class HttpClient{
 private BufferedReader inBuf;
 private PrintWriter outBuf;
 private Socket sock;
 private int bodySize;
 private String body;
 private URL url;
 private String statusCode;
 private String portName;

 /**
 * Calculates the "Content-Length" field of the request HTTP header.
 * (Each character is 1 byte long in UTF-8)
 */
 protected int getContentLength(){
  return body.length();
 }

 /**
 * Returns the SOAP-HTTP POST header
 * <lu>
 * <li>Content-Type is set to text/xml (XML specifications)-</li>
 * <li>charset is set to utf-8 (XML specifications)-</li>
 * <li>SOAPAction is set to the destination SOAP port (SOAP specifications)</li>
 * </lu>
 */
 protected String getHeader(){
  return "POST "+
         url.getPath()+
         " HTTP/1.1\nContent-Type: text/xml; charset=\"UNICODE-1-1-UTF-8\"\nContent-Length: "+
         getContentLength()+
         "\nSOAPAction: \""+
         portName+
         "\"\r\n\r\n";
 }

 /**
 * Returns the "Status-Code" field of the response HTTP header.
 */
 public String getStatusCode(){
  return statusCode;
 }

 /**
 * connects to the URI uri, and returns the body of the HTTP response.
 * See <a href="http://www.ietf.org/rfc/rfc2068.txt">http://www.ietf.org/rfc/rfc2068.txt</a>
 * for the HTTP 1.1 RFC.
 */
 public String getHttpResource(String urlStr,String portName,String body){
  String respBody=null;
  this.body=body;
  try{
   this.portName=portName;
   url=new URL(urlStr);
   sock=new Socket(url.getHost(),url.getPort());
   outBuf=new PrintWriter(new BufferedWriter(new OutputStreamWriter(sock.getOutputStream())),false);
   inBuf=new BufferedReader(new InputStreamReader(sock.getInputStream()));
   outBuf.print(getHeader());
   outBuf.print(body);
   outBuf.flush();
   // first line of the HTTP response header is
   // a return status code
   statusCode=inBuf.readLine();
   int ch;
   StringBuffer sBuf=new StringBuffer();
   while((ch=inBuf.read())!=-1)sBuf.append((char)ch);
   respBody=sBuf.toString();
   respBody=respBody.substring(respBody.indexOf("\r\n\r\n")+4,respBody.length());
  }
  catch(IOException e){
   e.printStackTrace();
  }
  finally{
   try{
    if(sock!=null)sock.close();
   }
   catch(IOException e){}
  }
  return respBody;
 }

 /**
 * For testing only
 */
 public static void main(String args[]){
  System.out.println((new HttpClient()).getHttpResource("http://planck:8080/xmlbroker/xb1",
   "http://www.jmrosengard.com/xmlbroker#demo",
   " <SOAP-ENV:Envelope"+
   " xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\""+
   " SOAP-ENV:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"+
   " <SOAP-ENV:Body>"+
   " <demo:getNtpTime xmlns:demo=\"http://www.jmrosengard.com/xmlbroker/demo\">"+
   " </demo:getNtpTime>"+
   " </SOAP-ENV:Body>"+
   " </SOAP-ENV:Envelope>"));
 }
}
