
package com.jmrosengard.xmlbroker.broker;

import java.io.*;
import java.util.*;
import javax.servlet.http.*;
import com.jmrosengard.xmlbroker.soapipr.Method;
import com.jmrosengard.xmlbroker.soapipr.SoapInterpreter;
import com.jmrosengard.xmlbroker.soapipr.RespMsg;
import com.jmrosengard.xmlbroker.javaipr.Linker;
import com.jmrosengard.xmlbroker.javaipr.JavaIprException;

/**
* Dispatches the HTTP requests to the relevant Port, based
* on the <i>SOAPAction</i> HTTP-SOAP header field.
*/

public class RequestMultiplexer extends HttpConnector{
 protected PortRegistry reg;

 public RequestMultiplexer(){
  reg=PortRegistry.getDefaultRegistry();
 }

 /**
 * Dispatch the SOAP request to an instance of the
 * relevant Port, and process the method call and
 * response.
 */
 public void doSoapPost(String targetUri,BufferedReader inBuf,PrintWriter outBuf)
 throws IOException,SoapReqException{
  Port port;
  PortKey key;
  if(targetUri!=null)port=reg.getPort(key=new PortKey(targetUri));
  else throw new SoapReqException(SOAP_HTTP_HEADER_SOAPACTION+
   " mandatory HTTP header field is missing.");
  if(port==null)throw new SoapReqException(
   "Invalid service requested: "+key);
  System.out.println("[XMLbroker] SOAP request received for port "+key);
  // build `virtual' method signature from SOAP request
  Method virtual=(new SoapInterpreter(inBuf)).parse();
  try{
   // execute method call
   (new Linker(port,virtual)).invoke();
  }catch(JavaIprException e){
   e.printStackTrace();
  }
  // Generate SOAP response
  RespMsg resp=new RespMsg(outBuf,virtual.getName(),key);
  resp.setParams(virtual.getOutParams());
  resp.build();
 }
}