package com.jmrosengard.xmlbroker.javaipr;

import com.jmrosengard.xmlbroker.soapipr.*;
import com.jmrosengard.xmlbroker.broker.Port;
import com.jmrosengard.xmlbroker.deploy.XbString;
import java.util.ArrayList;

/**
* Link the `virtual' Method object-- representing the method as invoked
* by the SOAP request message with the method implementation (a.k.a. servant).
* Another Method object, representing the actual method
* as defined by the service interface, defines the parameter passing
* semantics of this operation.
*/

public class Linker{
 private Port port;
 private Method actual;
 private Method virtual;
 private Object servant;
 private XbString[] paramQueue;

 public Linker(Port port,Method virtual){
  this.port=port;
  this.virtual=virtual;
  actual=port.getActualMtd();
  servant=port.getServant();
 }

 /**
 * Invoke the method on the servant object
 */
 public synchronized void invoke() throws JavaIprException{
  String mtdName=actual.getName();
  try{
   java.lang.reflect.Method target;
   int cnt=actual.getParamCount();
   Class[] formals=new Class[cnt];
   for(int i=0;i<cnt;i++)formals[i]=XbString.class;
   // create a java.lang.reflect.Method
   target=(servant.getClass()).getMethod(mtdName,formals);
   int inCnt=actual.getInParams().size();
   int outCnt=actual.getOutParams().size();
   // initialise the `outs' in virtual
   for(int i=0;i<outCnt;i++)virtual.setParam(new Parameter(
    "outParam"+(i+1),
    new Mode(Mode.MODE_OUT),
    new XbString()));
   ArrayList ins=virtual.getInParams();
   ArrayList outs=virtual.getOutParams();
   paramQueue=new XbString[cnt];
   for(int i=0;i<inCnt;paramQueue[i]=
    ((Parameter)ins.get(i++)).getValue());
   for(int i=0;i<outCnt;paramQueue[i+inCnt]=
    ((Parameter)outs.get(i++)).getValue());
   // invoke method !!
   target.invoke(servant,paramQueue);
   virtual.setReturned();
   System.out.println("[XMLbroker] OUT parameters returned from method call:");
   Parameter p;
   for(int i=0;i<cnt;i++)System.out.println("[XMLbroker] "+
    (p=(Parameter)virtual.getOutParams().get(i)).getName()+
    " = "+p.getValue().toString());
  }
  catch(Exception e){
   e.printStackTrace();
  throw new JavaIprException("An error occurred when invoking: "+mtdName);
  }
 }
}