package com.jmrosengard.xmlbroker.soapipr;

import org.w3c.dom.*;
import javax.xml.parsers.*;
import java.io.*;

/**
* Build a SOAP message as a DOM tree, allow
* subsclasses to add elements to the tree, and
* serialise the tree into a PirintWriter.
*/

public abstract class Message{
 protected static final String SOAP_ENV_NS_URI="http://schemas.xmlsoap.org/soap/envelope/";
 protected static final String SOAP_ENC_URI="http://schemas.xmlsoap.org/soap/encoding/";
 protected static final String XML_NS_URI= "http://www.w3.org/2000/xmlns/";
 protected static final String XB_NS_URI_BASE="http://www.jmrosengard.com/xmlbroker/";

 private PrintWriter outBuf;
 protected Document doc;
 protected Element root,body,meth,param;

 public Message(PrintWriter outBuf){
  this.outBuf=outBuf;
  try{
   DocumentBuilderFactory dbf =DocumentBuilderFactory.newInstance();
   DocumentBuilder db = dbf.newDocumentBuilder();
   doc = db.newDocument();
  }
  catch(ParserConfigurationException e){
   e.printStackTrace();
  }
 }

 /**
 * Hook method (See Template Pattern in [Gamma,E. & al. 1995])
 * that generates the actual SOAP message content.
 */
 protected abstract void doContent();

 /**
 * Create the generic parts of the message
 * and delegate the specific parts to
 * the hook method of the different subclasses.
 */
 public void build(){
  try{
   root=doc.createElementNS(SOAP_ENV_NS_URI,"SOAP-ENV:Envelope");
   root.setAttributeNS(SOAP_ENV_NS_URI,"SOAP-ENV:encodingStyle",SOAP_ENC_URI);
   root.setAttributeNS(XML_NS_URI,"xmlns:SOAP-ENV",SOAP_ENV_NS_URI);
   root.appendChild(doc.createTextNode("\n"));
   doc.appendChild(root);
   body=doc.createElementNS(SOAP_ENV_NS_URI,"SOAP-ENV:Body");
   body.appendChild(doc.createTextNode("\n"));
   root.appendChild(body);
   root.appendChild(doc.createTextNode("\n"));
   doContent();  // call hook method
   serialise(doc);
  }
  catch(DOMException e){
   e.printStackTrace();
  }
 }

 /**
 * Serialise the SOAP message's DOM tree
 * into a stream.
 */
 protected void serialise(Node node){
  int type = node.getNodeType();
  switch(type){
   case Node.ELEMENT_NODE:{
    outBuf.print('<' + node.getNodeName() );
    NamedNodeMap nnm = node.getAttributes();
    if(nnm != null ){
     int len = nnm.getLength() ;
     Attr attr;
     for ( int i = 0; i < len; i++ ){
      attr = (Attr)nnm.item(i);
       outBuf.print(' '
       + attr.getNodeName()
       + "=\""
       + attr.getNodeValue()
       +  '"' );
     }
    }
    outBuf.print('>');
    break;
   }
   case Node.TEXT_NODE:{
    outBuf.print(node.getNodeValue());
    break;
   }
  }
  // recursive call
  for(Node child=node.getFirstChild();child!=null;child=child.getNextSibling()){
   serialise(child);
  }
  // terminate last element
  if ( type == Node.ELEMENT_NODE ){
   outBuf.print("</" + node.getNodeName() + ">");
  }
 }

 /**
 * Only for testing
 */
 /*public static void main(String[] a){
  Message m=new Message(null);
  m.build();
 }*/
}