package com.jmrosengard.xmlbroker.soapipr;

import java.util.*;
import com.jmrosengard.xmlbroker.deploy.XbString;
/**
* Represents a method, according the model:<br>
* <tt>methodName__in__out(param1,param2)</tt>
*/

 public class Method{
 private String name;
 private ArrayList params;
 private boolean hasReturned;

 public Method(String name){
  this.name=name;
  params=new ArrayList(5);
 }

 /**
 * Bind a parameter to the method
 */
 public synchronized void setParam(Parameter param) throws SoapInterpreterException{
  if(! params.contains(param)) params.add(param);
  else throw new SoapInterpreterException("Duplicate parameter: "+param.getName());
 }

 /**
 * Return a ArrayList with all the `out' parameters
 */
 public synchronized ArrayList getOutParams(){
  ArrayList outs=new ArrayList(5);
  Iterator it=params.iterator();
  Parameter p;
  while(it.hasNext())
   if((p=(Parameter)it.next()).getMode().isOut())
   outs.add(p);
  return outs;
 }

 /**
 * Return a ArrayList with all the `in' parameters
 */
 public synchronized ArrayList getInParams(){
  ArrayList ins=new ArrayList(5);
  Iterator it=params.iterator();
  Parameter p;
  while(it.hasNext())
   if(!(p=(Parameter)it.next()).getMode().isOut())
   ins.add(p);
  return ins;
 }
 /**
 * Return the number of parameters
 */
 public int getParamCount(){
  return params.size();
 }
 /**
 * Return an Iterator to the parameter list
 */
 public Iterator paramIterator(){
  return params.iterator();
 }

 /**
 * Return the method's name
 */
 public String getName(){
  return name;
 }
 /**
 * Return <tt>true</tt> if the corresponding
 * method has been called and returned successfully
 */
 public boolean hasReturned(){
  return hasReturned;
 }
 /**
 * Call this method to set the return status of the
 * method represented by <tt>this</tt> as true.
 */
 public void setReturned(){
  hasReturned=true;
 }
}