package com.jmrosengard.xmlbroker.soapipr;

/**
* Models a parameter passing Mode (<tt>in</tt>
* as an `enumeration'.
* or <tt>out</tt>)
*/

public class Mode implements Comparable{

 public static final short MODE_IN=0;
 public static final short MODE_OUT=1;
 public static final String IN_MARK="__in";
 public static final String OUT_MARK="__out";

 // could allow MODE_IN_OUT in a future iteration
 private short mode;

 public Mode(short mode){
  if(mode==MODE_IN || mode==MODE_OUT) this.mode=mode;
 }

 /**
 * Return <tt>true</tt> if the two <tt>Mode</tt>s are equal
 */
 public boolean equals(Object o){
  return o instanceof Mode && ((Mode)o).getValue()==mode;
 }

 /**
 * Provided for sorting collections of Mode objects
 */
 public int compareTo(Object o){
  return ((Mode)o).toString().compareTo(toString());
 }

 /**
 * return the current mode
 */
 public short getValue(){
  return mode;
 }
 /**
 * Return <tt>true</tt> if mode==MODE_OUT
 */
 public boolean isIn(){
  return mode==MODE_IN;
 }

 /**
 * Return <tt>true</tt> if mode==MODE_IN
 */
 public boolean isOut(){
  return mode==MODE_OUT;
 }

 /**
 * return the mark corresponding to
 * the current mode
 */
 public String getMark(){
  return mode==MODE_IN?IN_MARK:OUT_MARK;
 }

 /**
 * return a String representation of
 * the current mode
 */
 public String toString(){
  return mode==MODE_IN?"in":"out";
 }
}
